@@ -1,7 +1,7 @@
 /* uuchk.c
    Display what we think the permissions of systems are.
 
-   Copyright (C) 1991, 1992 Ian Lance Taylor
+   Copyright (C) 1991, 1992, 1993, 1994, 1995 Ian Lance Taylor
 
    This file is part of the Taylor UUCP package.
 
@@ -17,16 +17,16 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    The author of the program may be contacted at ian@airs.com or
-   c/o Infinity Development Systems, P.O. Box 520, Waltham, MA 02254.
+   c/o Cygnus Support, 48 Grove Street, Somerville, MA 02144.
    */
 
 #include "uucp.h"
 
 #if USE_RCS_ID
-const char uuchk_rcsid[] = "$Id: uuchk.c,v 1.37 1992/10/16 05:10:26 ian Rel $";
+const char uuchk_rcsid[] = "$Id: uuchk.c,v 1.65 1995/07/18 00:59:30 ian Rel $";
 #endif
 
 #include "getopt.h"
@@ -36,6 +36,7 @@ const char uuchk_rcsid[] = "$Id: uuchk.c,v 1.37 1992/10/16 05:10:26 ian Rel $";
 /* Local functions.  */
 
 static void ukusage P((void));
+static void ukhelp P((void));
 static void ukshow P((const struct uuconf_system *qsys,
 		      pointer puuconf));
 static int ikshow_port P((struct uuconf_port *qport, pointer pinfo));
@@ -44,6 +45,7 @@ static void ukshow_chat P((const struct uuconf_chat *qchat,
 			   const char *zhdr));
 static void ukshow_size P((struct uuconf_timespan *q, boolean fcall,
 			   boolean flocal));
+static void ukshow_reliable P ((int i, const char *zhdr));
 static void ukshow_proto_params P((struct uuconf_proto_param *pas,
 				   int cindent));
 static void ukshow_time P((const struct uuconf_timespan *));
@@ -62,8 +64,19 @@ struct sinfo
   boolean fgot;
 };
 
+/* Program name.  */
+static const char *zKprogram;
+
 /* Long getopt options.  */
-static const struct option asKlongopts[] = { { NULL, 0, NULL, 0 } };
+static const struct option asKlongopts[] =
+{
+  { "system", required_argument, NULL,'s' },
+  { "config", required_argument, NULL, 'I' },
+  { "debug", required_argument, NULL, 'x' },
+  { "version", no_argument, NULL, 'v' },
+  { "help", no_argument, NULL, 1 },
+  { NULL, 0, NULL, 0 }
+};
 
 int
 main (argc, argv)
@@ -71,17 +84,23 @@ main (argc, argv)
      char **argv;
 {
   int iopt;
-  /* The configuration file name.  */
+  const char *zsystem = NULL;
   const char *zconfig = NULL;
   int iret;
   pointer puuconf;
-  char **pzsystems;
 
-  while ((iopt = getopt_long (argc, argv, "I:x:", asKlongopts,
+  zKprogram = argv[0];
+
+  while ((iopt = getopt_long (argc, argv, "I:s:vx:", asKlongopts,
 			      (int *) NULL)) != EOF)
     {
       switch (iopt)
 	{
+	case 's':
+	  /* Examine specific system.  */
+	  zsystem = optarg;
+	  break;
+
 	case 'I':
 	  /* Set the configuration file name.  */
 	  zconfig = optarg;
@@ -92,40 +111,165 @@ main (argc, argv)
 	     information for this program.  */
 	  break;
 
+	case 'v':
+	  /* Print version and exit.  */
+	  printf ("%s: Taylor UUCP %s, copyright (C) 1991, 92, 93, 94, 1995 Ian Lance Taylor\n",
+		  zKprogram, VERSION);
+	  exit (EXIT_SUCCESS);
+	  /*NOTREACHED*/
+	  
+	case 1:
+	  /* --help.  */
+	  ukhelp ();
+	  exit (EXIT_SUCCESS);
+	  /*NOTREACHED*/
+
 	case 0:
 	  /* Long option found and flag set.  */
 	  break;
 
 	default:
 	  ukusage ();
-	  break;
+	  /*NOTREACHED*/
 	}
     }
 
   if (optind != argc)
-    ukusage ();
+    {
+      fprintf (stderr, "%s: too many arguments", zKprogram);
+      ukusage ();
+    }
 
   iret = uuconf_init (&puuconf, (const char *) NULL, zconfig);
   if (iret != UUCONF_SUCCESS)
     ukuuconf_error (puuconf, iret);
 
-  iret = uuconf_system_names (puuconf, &pzsystems, FALSE);
-  if (iret != UUCONF_SUCCESS)
-    ukuuconf_error (puuconf, iret);
-
-  while (*pzsystems != NULL)
+  if (zsystem != NULL)
     {
       struct uuconf_system ssys;
 
-      iret = uuconf_system_info (puuconf, *pzsystems, &ssys);
+      iret = uuconf_system_info (puuconf, zsystem, &ssys);
+      if (iret == UUCONF_NOT_FOUND)
+	{
+	  fprintf (stderr, "%s: system not found\n", zsystem);
+	  exit (EXIT_FAILURE);
+	}
+      else if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+
+      ukshow (&ssys, puuconf);
+      (void) uuconf_system_free (puuconf, &ssys);
+    }
+  else
+    {
+      const char *zstr;
+      int iint;
+      char **pzsystems;
+
+      iret = uuconf_localname (puuconf, &zstr);
+      if (iret == UUCONF_SUCCESS)
+	printf ("Local node name %s\n", zstr);
+      else if (iret != UUCONF_NOT_FOUND)
+	ukuuconf_error (puuconf, iret);
+
+      iret = uuconf_spooldir (puuconf, &zstr);
       if (iret != UUCONF_SUCCESS)
 	ukuuconf_error (puuconf, iret);
+      printf ("Spool directory %s\n", zstr);
+
+      iret = uuconf_pubdir (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("Public directory %s\n", zstr);
+
+      iret = uuconf_lockdir (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("Lock directory %s\n", zstr);
+
+      iret = uuconf_logfile (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("Log file %s\n", zstr);
+
+      iret = uuconf_statsfile (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("Statistics file %s\n", zstr);
+
+      iret = uuconf_debugfile (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("Debug file %s\n", zstr);
+
+      iret = uuconf_debuglevel (puuconf, &zstr);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      if (zstr != NULL)
+	printf ("Global debugging level %s\n", zstr);
+
+      iret = uuconf_strip (puuconf, &iint);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      printf ("uucico -l will %sstrip login names and passwords\n",
+	      (iint & UUCONF_STRIP_LOGIN) != 0 ? "" : "not ");
+      printf ("uucico will %sstrip UUCP protocol commands\n",
+	      (iint & UUCONF_STRIP_PROTO) != 0 ? "" : "not ");
+
+      iret = uuconf_maxuuxqts (puuconf, &iint);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      if (iint != 0)
+	printf ("Maximum number of uuxqt processes permitted %d\n", iint);
+
+      iret = uuconf_runuuxqt (puuconf, &iint);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+      if (iint > 0)
+	printf ("Start uuxqt every %d jobs\n", iint);
       else
-	ukshow (&ssys, puuconf);
-      (void) uuconf_system_free (puuconf, &ssys);
-      ++pzsystems;
-      if (*pzsystems != NULL)
-	printf ("\n");
+	{
+	  switch (iint)
+	    {
+	    case UUCONF_RUNUUXQT_NEVER:
+	      printf ("Never start uuxqt\n");
+	      break;
+	    case UUCONF_RUNUUXQT_ONCE:
+	      printf ("Start uuxqt once per uucico invocation\n");
+	      break;
+	    case UUCONF_RUNUUXQT_PERCALL:
+	      printf ("Start uuxqt once per call\n");
+	      break;
+	    default:
+	      fprintf (stderr, "Illegal value from uuconf_runuuxqt\n");
+	      exit (EXIT_FAILURE);
+	      break;
+	    }
+	}
+
+      iret = uuconf_system_names (puuconf, &pzsystems, FALSE);
+      if (iret != UUCONF_SUCCESS)
+	ukuuconf_error (puuconf, iret);
+
+      if (*pzsystems == NULL)
+	{
+	  fprintf (stderr, "%s: no systems found\n", zKprogram);
+	  exit (EXIT_FAILURE);
+	}
+
+      while (*pzsystems != NULL)
+	{
+	  struct uuconf_system ssys;
+
+	  printf ("\n");
+	  iret = uuconf_system_info (puuconf, *pzsystems, &ssys);
+	  if (iret != UUCONF_SUCCESS)
+	    ukuuconf_error (puuconf, iret);
+	  else
+	    ukshow (&ssys, puuconf);
+	  (void) uuconf_system_free (puuconf, &ssys);
+	  ++pzsystems;
+	}
     }
 
   exit (EXIT_SUCCESS);
@@ -136,18 +280,26 @@ main (argc, argv)
 
 /* Print a usage message and die.  */
 
-static void
-ukusage ()
+static void ukusage ()
 {
-  fprintf (stderr,
-	   "Taylor UUCP version %s, copyright (C) 1991, 1992 Ian Lance Taylor\n",
-	   VERSION);
-  fprintf (stderr,
-	   "Usage: uuchk [-I file]\n");
-  fprintf (stderr,
-	   " -I file: Set configuration file to use\n");
+  fprintf (stderr, "Usage: %s [-s system] [-I file]\n", zKprogram);
+  fprintf (stderr, "Use %s --help for help\n", zKprogram);
   exit (EXIT_FAILURE);
 }
+
+/* Print a help message.  */
+
+static void
+ukhelp ()
+{
+  printf ("Taylor UUCP %s, copyright (C) 1991, 92, 93, 94, 1995 Ian Lance Taylor\n",
+	  VERSION);
+  printf ("Usage: %s [-s system] [-I file] [-v]\n", zKprogram);
+  printf (" -s,--system system: Only print configuration for named system\n");
+  printf (" -I,--config file: Set configuration file to use\n");
+  printf (" -v,--version: Print version and exit\n");
+  printf (" --help: Print help and exit\n");
+}
 
 /* Dump out the information for a system.  */
 
@@ -159,6 +311,7 @@ ukshow (qsys, puuconf)
   char **pz;
   int i;
   int iret;
+  boolean fanycall;
 
   printf ("System: %s", qsys->uuconf_zname);
   if (qsys->uuconf_pzalias != NULL)
@@ -174,6 +327,7 @@ ukshow (qsys, puuconf)
     }
   printf ("\n");
 
+  fanycall = FALSE;
   for (i = 0; qsys != NULL; qsys = qsys->uuconf_qalternate, i++)
     {
       boolean fcall, fcalled;
@@ -201,6 +355,9 @@ ukshow (qsys, puuconf)
 	  continue;
 	}
 
+      if (fcall)
+	fanycall = TRUE;
+
       if (fcalled)
 	{
 	  if (qsys->uuconf_zcalled_login != NULL
@@ -310,6 +467,8 @@ ukshow (qsys, puuconf)
 	      iret = uuconf_callout (puuconf, qsys, &zlogin, &zpass);
 	      if (iret == UUCONF_NOT_FOUND)
 		printf (" Can not determine login name or password\n");
+	      else if (UUCONF_ERROR_VALUE (iret) == UUCONF_FOPEN_FAILED)
+		printf (" Can not read call out file\n");
 	      else if (iret != UUCONF_SUCCESS)
 		ukuuconf_error (puuconf, iret);
 	      else
@@ -343,6 +502,10 @@ ukshow (qsys, puuconf)
 	      printf ("\n");
 	    }
 
+	  if (qsys->uuconf_cmax_retries > 0)
+	    printf (" May retry the call up to %d times\n",
+		    qsys->uuconf_cmax_retries);
+
 	  if (qsys->uuconf_qcalltimegrade != NULL)
 	    {
 	      boolean fprint, fother;
@@ -381,6 +544,50 @@ ukshow (qsys, puuconf)
 	    }
 	}
 
+      if (fcalled)
+	{
+	  if (qsys->uuconf_qcalledtimegrade != NULL)
+	    {
+	      boolean fprint, fother;
+
+	      qtime = qcompress_span (qsys->uuconf_qcalledtimegrade);
+	      fprint = FALSE;
+	      fother = FALSE;
+	      if (qtime->uuconf_istart != 0)
+		fother = TRUE;
+	      for (qspan = qtime; qspan != NULL; qspan = qspan->uuconf_qnext)
+		{
+		  if ((char) qspan->uuconf_ival == UUCONF_GRADE_LOW)
+		    {
+		      fother = TRUE;
+		      continue;
+		    }
+		  fprint = TRUE;
+		  printf (" ");
+		  ukshow_time (qspan);
+		  printf (" will send work grade %c or higher\n",
+			  (char) qspan->uuconf_ival);
+		  if (qspan->uuconf_qnext == NULL)
+		    {
+		      if (qspan->uuconf_iend != 7 * 24 * 60)
+			fother = TRUE;
+		    }
+		  else
+		    {
+		      if (qspan->uuconf_iend
+			  != qspan->uuconf_qnext->uuconf_istart)
+			fother = TRUE;
+		    }
+		}
+	      if (fprint && fother)
+		printf (" (At other times will send any work)\n");
+	    }
+	}
+
+      if (fcall && qsys->uuconf_csuccess_wait != 0)
+	printf (" Will wait %d seconds after a successful call\n",
+		qsys->uuconf_csuccess_wait);
+
       if (qsys->uuconf_fsequence)
 	printf (" Sequence numbers are used\n");
 
@@ -401,7 +608,7 @@ ukshow (qsys, puuconf)
       if (fcalled)
 	{
 	  ukshow_size (qsys->uuconf_qcalled_local_size, FALSE, TRUE);
-	  ukshow_size (qsys->uuconf_qcalled_remote_size, FALSE, TRUE);
+	  ukshow_size (qsys->uuconf_qcalled_remote_size, FALSE, FALSE);
 	}
 
       if (fcall)
@@ -485,6 +692,9 @@ ukshow (qsys, puuconf)
       if (qsys->uuconf_qproto_params != NULL)
 	ukshow_proto_params (qsys->uuconf_qproto_params, 1);
     }
+
+  if (! fanycall)
+    printf (" Calls will never be placed to this system\n");
 }
 
 /* Show information about a port.  */
@@ -498,7 +708,9 @@ ikshow_port (qport, pinfo)
   struct sinfo *qi = (struct sinfo *) pinfo;
   char **pz;
   struct uuconf_modem_port *qmodem;
+  struct uuconf_tcp_port *qtcp;
   struct uuconf_tli_port *qtli;
+  struct uuconf_pipe_port *qpipe;
 
   qi->fgot = TRUE;
 
@@ -513,13 +725,21 @@ ikshow_port (qport, pinfo)
       if (qport->uuconf_u.uuconf_sdirect.uuconf_zdevice != NULL)
 	printf ("   Device %s\n",
 		qport->uuconf_u.uuconf_sdirect.uuconf_zdevice);
+      else
+	printf ("   Using port name as device name\n");
       printf ("   Speed %ld\n", qport->uuconf_u.uuconf_sdirect.uuconf_ibaud);
+      printf ("   Carrier %savailable\n",
+	      qport->uuconf_u.uuconf_sdirect.uuconf_fcarrier ? "" : "not ");
+      printf ("   Hardware flow control %savailable\n",
+	      qport->uuconf_u.uuconf_sdirect.uuconf_fhardflow ? "" : "not ");
       break;
     case UUCONF_PORTTYPE_MODEM:
       qmodem = &qport->uuconf_u.uuconf_smodem;
       printf ("   Port type modem\n");
       if (qmodem->uuconf_zdevice != NULL)
 	printf ("   Device %s\n", qmodem->uuconf_zdevice);
+      else
+	printf ("   Using port name as device name\n");
       if (qmodem->uuconf_zdial_device != NULL)
 	printf ("   Dial device %s\n", qmodem->uuconf_zdial_device);
       printf ("   Speed %ld\n", qmodem->uuconf_ibaud);
@@ -528,6 +748,8 @@ ikshow_port (qport, pinfo)
 		qmodem->uuconf_ihighbaud);
       printf ("   Carrier %savailable\n",
 	      qmodem->uuconf_fcarrier ? "" : "not ");
+      printf ("   Hardware flow control %savailable\n",
+	      qmodem->uuconf_fhardflow ? "" : "not ");
       if (qmodem->uuconf_qdialer != NULL)
 	{
 	  printf ("   Specially defined dialer\n");
@@ -589,9 +811,17 @@ ikshow_port (qport, pinfo)
 	printf ("   *** No dialer information\n");
       break;
     case UUCONF_PORTTYPE_TCP:
+      qtcp = &qport->uuconf_u.uuconf_stcp;
       printf ("   Port type tcp\n");
-      printf ("   TCP service %s\n",
-	      qport->uuconf_u.uuconf_stcp.uuconf_zport);
+      printf ("   TCP service %s\n", qtcp->uuconf_zport);
+      if (qtcp->uuconf_pzdialer != NULL
+	  && qtcp->uuconf_pzdialer[0] != NULL)
+	{
+	  printf ("   Dialer sequence");
+	  for (pz = qtcp->uuconf_pzdialer; *pz != NULL; pz++)
+	    printf (" %s", *pz);
+	  printf ("\n");
+	}
       break;
     case UUCONF_PORTTYPE_TLI:
       qtli = &qport->uuconf_u.uuconf_stli;
@@ -599,6 +829,8 @@ ikshow_port (qport, pinfo)
 	      qtli->uuconf_fstream ? "S" : "");
       if (qtli->uuconf_zdevice != NULL)
 	printf ("   Device %s\n", qtli->uuconf_zdevice);
+      else
+	printf ("   Using port name as device name\n");
       if (qtli->uuconf_pzpush != NULL)
 	{
 	  printf ("   Push");
@@ -617,6 +849,17 @@ ikshow_port (qport, pinfo)
       if (qtli->uuconf_zservaddr != NULL)
 	printf ("   Server address %s\n", qtli->uuconf_zservaddr);
       break;
+    case UUCONF_PORTTYPE_PIPE:
+      qpipe = &qport->uuconf_u.uuconf_spipe;
+      printf ("   Port type pipe\n");
+      if (qpipe->uuconf_pzcmd != NULL)
+	{
+	  printf ("   Command");
+	  for (pz = qpipe->uuconf_pzcmd; *pz != NULL; pz++)
+	    printf (" %s", *pz);
+	  printf ("\n");
+	}
+      break;
     default:
       fprintf (stderr, "   CAN'T HAPPEN\n");
       break;
@@ -628,6 +871,9 @@ ikshow_port (qport, pinfo)
   if (qport->uuconf_zlockname != NULL)
     printf ("   Will use lockname %s\n", qport->uuconf_zlockname);
 
+  if ((qport->uuconf_ireliable & UUCONF_RELIABLE_SPECIFIED) != 0)
+    ukshow_reliable (qport->uuconf_ireliable, "   ");
+
   if (qport->uuconf_qproto_params != NULL)
     ukshow_proto_params (qport->uuconf_qproto_params, 3);
 
@@ -656,6 +902,8 @@ ukshow_dialer (q)
     }
   ukshow_chat (&q->uuconf_scomplete, "    When complete chat");
   ukshow_chat (&q->uuconf_sabort, "    When aborting chat");
+  if ((q->uuconf_ireliable & UUCONF_RELIABLE_SPECIFIED) != 0)
+    ukshow_reliable (q->uuconf_ireliable, "   ");
   if (q->uuconf_qproto_params != NULL)
     ukshow_proto_params (q->uuconf_qproto_params, 4);
 }
@@ -744,6 +992,29 @@ ukshow_size (qspan, fcall, flocal)
     printf ("  (At other times may send files of any size)\n");
 }
 
+/* Show reliability information.  */
+
+static void
+ukshow_reliable (i, zhdr)
+     int i;
+     const char *zhdr;
+{
+  printf ("%sCharacteristics:", zhdr);
+  if ((i & UUCONF_RELIABLE_EIGHT) != 0)
+    printf (" eight-bit-clean");
+  else
+    printf (" not-eight-bit-clean");
+  if ((i & UUCONF_RELIABLE_RELIABLE) != 0)
+    printf (" reliable");
+  if ((i & UUCONF_RELIABLE_ENDTOEND) != 0)
+    printf (" end-to-end");
+  if ((i & UUCONF_RELIABLE_FULLDUPLEX) != 0)
+    printf (" fullduplex");
+  else
+    printf (" halfduplex");
+  printf ("\n");
+}
+
 /* Show protocol parameters.  */
 
 static void
@@ -849,8 +1120,8 @@ ukuuconf_error (puuconf, iret)
 
   (void) uuconf_error_string (puuconf, iret, ab, sizeof ab);
   if ((iret & UUCONF_ERROR_FILENAME) == 0)
-    fprintf (stderr, "uuchk: %s\n", ab);
+    fprintf (stderr, "%s: %s\n", zKprogram, ab);
   else
-    fprintf (stderr, "uuchk:%s\n", ab);
+    fprintf (stderr, "%s:%s\n", zKprogram, ab);
   exit (EXIT_FAILURE);
 }
