@@ -17,16 +17,16 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    The author of the program may be contacted at ian@airs.com or
-   c/o Cygnus Support, Building 200, 1 Kendall Square, Cambridge, MA 02139.
+   c/o Cygnus Support, 48 Grove Street, Somerville, MA 02144.
    */
 
 #include "uucnfi.h"
 
 #if USE_RCS_ID
-const char _uuconf_cmdlin_rcsid[] = "$Id: cmdlin.c,v 1.3 1994/01/30 21:14:29 ian Rel $";
+const char _uuconf_cmdlin_rcsid[] = "$Id: cmdlin.c,v 1.6 1995/06/21 19:21:55 ian Rel $";
 #endif
 
 #include <errno.h>
@@ -56,18 +56,21 @@ uuconf_cmd_line (pglobal, zline, qtab, pinfo, pfiunknown, iflags, pblock)
   char **pzargs;
   int iret;
 
-  /* Any # not preceeded by a backslash starts a comment.  */
-  z = zline;
-  while ((z = strchr (z, '#')) != NULL)
+  if ((iflags & UUCONF_CMDTABFLAG_NOCOMMENTS) == 0)
     {
-      if (z == zline || *(z - 1) != '\\')
+      /* Any # not preceeded by a backslash starts a comment.  */
+      z = zline;
+      while ((z = strchr (z, '#')) != NULL)
 	{
-	  *z = '\0';
-	  break;
+	  if (z == zline || *(z - 1) != '\\')
+	    {
+	      *z = '\0';
+	      break;
+	    }
+	  /* Remove the backslash.  */
+	  while ((*(z - 1) = *z) != '\0')
+	    ++z;
 	}
-      /* Remove the backslash.  */
-      while ((*(z - 1) = *z) != '\0')
-	++z;
     }
 
   /* Parse the first CSTACK arguments by hand to avoid malloc.  */
