@@ -17,21 +17,21 @@
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    The author of the program may be contacted at ian@airs.com or
-   c/o Infinity Development Systems, P.O. Box 520, Waltham, MA 02254.
+   c/o Cygnus Support, 48 Grove Street, Somerville, MA 02144.
    */
 
 #include "uucnfi.h"
 
 #if USE_RCS_ID
-const char _uuconf_filnam_rcsid[] = "$Id: filnam.c,v 1.2 1992/06/15 18:40:43 ian Rel $";
+const char _uuconf_filnam_rcsid[] = "$Id: filnam.c,v 1.6 1995/06/21 19:22:20 ian Rel $";
 #endif
 
 /* Return the saved file name.  */
 
-const char *
+UUCONF_CONST char *
 uuconf_error_filename (pglobal)
      pointer pglobal;
 {
