@@ -1,7 +1,7 @@
 /* uux.c
    Prepare to execute a command on a remote system.
 
-   Copyright (C) 1991, 1992 Ian Lance Taylor
+   Copyright (C) 1991, 1992, 1993, 1994, 1995 Ian Lance Taylor
 
    This file is part of the Taylor UUCP package.
 
@@ -17,16 +17,16 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    The author of the program may be contacted at ian@airs.com or
-   c/o Infinity Development Systems, P.O. Box 520, Waltham, MA 02254.
+   c/o Cygnus Support, 48 Grove Street, Somerville, MA 02144.
    */
 
 #include "uucp.h"
 
 #if USE_RCS_ID
-const char uux_rcsid[] = "$Id: uux.c,v 1.56 1993/01/21 05:11:34 ian Rel $";
+const char uux_rcsid[] = "$Id: uux.c,v 1.82 1995/08/20 19:21:42 ian Rel $";
 #endif
 
 #include "uudefs.h"
@@ -54,11 +54,24 @@ const char uux_rcsid[] = "$Id: uux.c,v 1.56 1993/01/21 05:11:34 ian Rel $";
    operators.  */
 #define ZSHELLNONREDIRSEPS ";&*| \t"
 
-/* The program name.  */
-char abProgram[] = "uux";
+/* Whether this execution is occurring on the local system.  */
+static boolean fXxqtlocal;
 
-/* The name of the execute file.  */
-const char *zXxqt_name;
+/* The execution system.  */
+static struct uuconf_system sXxqtsys;
+
+/* The name of local system from the point of view of the execution
+   system.  */
+static const char *zXxqtloc;
+
+/* The job grade to use.  */
+static char bXgrade = BDEFAULT_UUX_GRADE;
+
+/* The temporary file name of the execute file.  */
+static char abXxqt_tname[CFILE_NAME_LEN];
+
+/* The name of the execute file on the remote system.  */
+static char abXxqt_xname[CFILE_NAME_LEN];
 
 /* The execute file we are creating.  */
 static FILE *eXxqt_file;
@@ -69,22 +82,44 @@ static int cXcmds;
 
 /* A file to close if we're forced to exit.  */
 static FILE *eXclose;
+
+/* A list of file names which will match the file names which appear
+   in the uucico logs.  */
+static char *zXnames;
 
 /* Local functions.  */
 static void uxusage P((void));
+static void uxhelp P((void));
 static void uxadd_xqt_line P((int bchar, const char *z1, const char *z2));
 static void uxadd_send_file P((const char *zfrom, const char *zto,
 			       const char *zoptions, const char *ztemp,
-			       const char *zforward,
-			       const struct uuconf_system *qxqtsys,
-			       const char *zxqtloc,
-			       int bgrade));
+			       const char *zforward));
 static void uxcopy_stdin P((FILE *e));
 static void uxrecord_file P((const char *zfile));
 static void uxabort P((void));
+static void uxadd_name P((const char *));
 
 /* Long getopt options.  */
-static const struct option asXlongopts[] = { { NULL, 0, NULL, 0 } };
+static const struct option asXlongopts[] =
+{
+  { "requestor", required_argument, NULL, 'a' },
+  { "return-stdin", no_argument, NULL, 'b' },
+  { "nocopy", no_argument, NULL, 'c' },
+  { "copy", no_argument, NULL, 'C' },
+  { "grade", required_argument, NULL, 'g' },
+  { "jobid", no_argument, NULL, 'j' },
+  { "link", no_argument, NULL, 'l' },
+  { "notification", required_argument, NULL, 2 },
+  { "stdin", no_argument, NULL, 'p' },
+  { "nouucico", no_argument, NULL, 'r' },
+  { "status", required_argument, NULL, 's' },
+  { "noexpand", no_argument, NULL, 'W' },
+  { "config", required_argument, NULL, 'I' },
+  { "debug", required_argument, NULL, 'x' },
+  { "version", no_argument, NULL, 'v' },
+  { "help", no_argument, NULL, 1 },
+  { NULL, 0, NULL, 0 }
+};
 
 /* The main routine.  */
 
@@ -106,8 +141,6 @@ main (argc, argv)
   const char *zconfig = NULL;
   /* -j: output job id.  */
   boolean fjobid = FALSE;
-  /* -g: job grade.  */
-  char bgrade = BDEFAULT_UUX_GRADE;
   /* -l: link file to spool directory.  */
   boolean flink = FALSE;
   /* -n: do not notify upon command completion.  */
@@ -126,7 +159,6 @@ main (argc, argv)
   pointer puuconf;
   int iuuconf;
   const char *zlocalname;
-  const char *zxqtloc;
   int i;
   size_t clen;
   char *zargs;
@@ -136,14 +168,10 @@ main (argc, argv)
   char *zexclam;
   boolean fgetcwd;
   const char *zuser;
-  struct uuconf_system sxqtsys;
-  boolean fxqtlocal;
   char *zforward;
   char **pzargs;
   int calloc_args;
   int cargs;
-  char abxqt_tname[CFILE_NAME_LEN];
-  char abxqt_xname[CFILE_NAME_LEN];
   const char *zinput_from;
   const char *zinput_to;
   const char *zinput_temp;
@@ -153,40 +181,32 @@ main (argc, argv)
   struct uuconf_system slocalsys;
   boolean fneedshell;
   char *zfullcmd;
+  boolean fpoll;
+  char aboptions[10];
   boolean fexit;
 
-  /* We need to be able to read a single - as an option, which getopt
-     won't do.  So that we can still use getopt, we run through the
-     options looking for an option "-"; if we find one we change it to
-     "-p", which is equivalent to "-".  */
-  for (i = 1; i < argc; i++)
-    {
-      if (argv[i][0] != '-')
-	break;
-      if (argv[i][1] == '\0')
-	argv[i] = zbufcpy ("-p");
-      else
-	{
-	  const char *z;
+  zProgram = argv[0];
 
-	  for (z = argv[i] + 1; *z != '\0'; z++)
-	    {
-	      /* If the option takes an argument, and the argument is
-		 not appended, then skip the next argument.  */
-	      if (*z == 'a' || *z == 'g' || *z == 'I'
-		  || *z == 's' || *z == 'x')
-		{
-		  if (z[1] == '\0')
-		    i++;
-		  break;
-		}
-	    }
-	}
+  /* We need to be able to read a single - as an option, which getopt
+     won't do.  We handle this by using getopt to scan the argument
+     list multiple times, replacing any single "-" with "-p".  */
+  opterr = 0;
+  while (1)
+    {
+      while (getopt_long (argc, argv, "+a:bcCg:I:jlnprs:Wvx:z",
+			  asXlongopts, (int *) NULL) != EOF)
+	;
+      if (optind >= argc || strcmp (argv[optind], "-") != 0)
+	break;
+      argv[optind] = zbufcpy ("-p");
+      optind = 0;
     }
+  opterr = 1;
+  optind = 0;
 
   /* The leading + in the getopt string means to stop processing
      options as soon as a non-option argument is seen.  */
-  while ((iopt = getopt_long (argc, argv, "+a:bcCg:I:jlnprs:Wx:z",
+  while ((iopt = getopt_long (argc, argv, "+a:bcCg:I:jlnprs:Wvx:z",
 			      asXlongopts, (int *) NULL)) != EOF)
     {
       switch (iopt)
@@ -226,7 +246,7 @@ main (argc, argv)
 
 	case 'g':
 	  /* Set job grade.  */
-	  bgrade = optarg[0];
+	  bXgrade = optarg[0];
 	  break;
 
 	case 'l':
@@ -271,6 +291,41 @@ main (argc, argv)
 	  ferror_ack = TRUE;
 	  break;
 
+	case 2:
+	  /* --notify={true,false,error}.  */
+	  if (*optarg == 't'
+	      || *optarg == 'T'
+	      || *optarg == 'y'
+	      || *optarg == 'Y'
+	      || *optarg == 'e'
+	      || *optarg == 'E')
+	    {
+	      ferror_ack = TRUE;
+	      fno_ack = FALSE;
+	    }
+	  else if (*optarg == 'f'
+		   || *optarg == 'F'
+		   || *optarg == 'n'
+		   || *optarg == 'N')
+	    {
+	      ferror_ack = FALSE;
+	      fno_ack = TRUE;
+	    }
+	  break;
+
+	case 'v':
+	  /* Print version and exit.  */
+	  printf ("%s: Taylor UUCP %s, copyright (C) 1991, 92, 93, 94, 1995 Ian Lance Taylor\n",
+		  zProgram, VERSION);
+	  exit (EXIT_SUCCESS);
+	  /*NOTREACHED*/
+
+	case 1:
+	  /* --help.  */
+	  uxhelp ();
+	  exit (EXIT_SUCCESS);
+	  /*NOTREACHED*/
+
 	case 0:
 	  /* Long option found and flag set.  */
 	  break;
@@ -281,10 +336,13 @@ main (argc, argv)
 	}
     }
 
-  if (! UUCONF_GRADE_LEGAL (bgrade))
+  if (! UUCONF_GRADE_LEGAL (bXgrade)
+      || ((bXgrade < '0' || bXgrade > '9')
+	  && (bXgrade < 'a' || bXgrade > 'z')
+	  && (bXgrade < 'A' || bXgrade > 'Z')))
     {
       ulog (LOG_ERROR, "Ignoring illegal grade");
-      bgrade = BDEFAULT_UUX_GRADE;
+      bXgrade = BDEFAULT_UUX_GRADE;
     }
 
   if (optind == argc)
@@ -351,7 +409,10 @@ main (argc, argv)
 					   calloc_args * sizeof (char *));
 	    }
 
-	  clen = strcspn (zarg, ZSHELLSEPS);
+	  if (*zarg == '(')
+	    clen = strlen (zarg);
+	  else
+	    clen = strcspn (zarg, ZSHELLSEPS);
 	  if (clen > 0)
 	    {
 	      pzargs[cargs] = zbufalc (clen + 1);
@@ -444,43 +505,24 @@ main (argc, argv)
       iuuconf = uuconf_system_local (puuconf, &slocalsys);
       if (iuuconf != UUCONF_SUCCESS)
 	ulog_uuconf (LOG_FATAL, puuconf, iuuconf);
+      slocalsys.uuconf_zname = (char *) zlocalname;
     }
 
-  /* Figure out which system the command is to be executed on.  Some
-     mailers apparently pass local!rmail, so we must explicitly check
-     for that.  */
+  /* Figure out which system the command is to be executed on.  */
+  zcmd = zremove_local_sys (&slocalsys, zcmd);
   zexclam = strchr (zcmd, '!');
-  while (zexclam != NULL)
-    {
-      *zexclam = '\0';
-      if (strcmp (zcmd, zlocalname) == 0)
-	;
-      else if (slocalsys.uuconf_pzalias == NULL)
-	break;
-      else
-	{
-	  char **pzal;
-
-	  for (pzal = slocalsys.uuconf_pzalias; *pzal != NULL; pzal++)
-	    if (strcmp (zcmd, *pzal) == 0)
-	      break;
-	  if (*pzal == NULL)
-	    break;
-	}
-      zcmd = zexclam + 1;
-      zexclam = strchr (zcmd, '!');
-    }
   if (zexclam == NULL)
     {
       zsys = zlocalname;
-      fxqtlocal = TRUE;
+      fXxqtlocal = TRUE;
       zforward = NULL;
     }
   else
     {
+      *zexclam = '\0';
       zsys = zcmd;
       zcmd = zexclam + 1;
-      fxqtlocal = FALSE;
+      fXxqtlocal = FALSE;
 
       /* See if we must forward this command through other systems
 	 (e.g. uux a!b!cmd).  */
@@ -497,40 +539,24 @@ main (argc, argv)
 	}
     }
 
-  if (fxqtlocal)
-    sxqtsys = slocalsys;
+  if (fXxqtlocal)
+    sXxqtsys = slocalsys;
   else
     {
-      iuuconf = uuconf_system_info (puuconf, zsys, &sxqtsys);
+      iuuconf = uuconf_system_info (puuconf, zsys, &sXxqtsys);
       if (iuuconf != UUCONF_SUCCESS)
 	{
 	  if (iuuconf != UUCONF_NOT_FOUND)
 	    ulog_uuconf (LOG_FATAL, puuconf, iuuconf);
-	  if (! funknown_system (puuconf, zsys, &sxqtsys))
+	  if (! funknown_system (puuconf, zsys, &sXxqtsys))
 	    ulog (LOG_FATAL, "%s: System not found", zsys);
 	}
     }
 
   /* Get the local name the remote system know us as.  */
-  zxqtloc = sxqtsys.uuconf_zlocalname;
-  if (zxqtloc == NULL)
-    zxqtloc = zlocalname;
-
-  /* We can send this as an E command if the execution is on a
-     different, directly connected, system and the only file used is
-     the standard input and comes from this system.  This is true of
-     the common cases of rmail and rnews.  We get an execute file name
-     here in case we need it.  */
-  if (fxqtlocal)
-    zXxqt_name = zsysdep_xqt_file_name ();
-  else
-    zXxqt_name = zsysdep_data_file_name (&sxqtsys, zxqtloc, bgrade, TRUE,
-					 abxqt_tname, (char *) NULL,
-					 abxqt_xname);
-  if (zXxqt_name == NULL)
-    uxabort ();
-
-  uxrecord_file (zXxqt_name);
+  zXxqtloc = sXxqtsys.uuconf_zlocalname;
+  if (zXxqtloc == NULL)
+    zXxqtloc = zlocalname;
 
   /* Look through the arguments.  Any argument containing an
      exclamation point character is interpreted as a file name, and is
@@ -587,6 +613,12 @@ main (argc, argv)
       if (zexclam == NULL && ! finput && ! foutput)
 	continue;
 
+      if (zexclam != NULL)
+	{
+	  pzargs[i] = zremove_local_sys (&slocalsys, pzargs[i]);
+	  zexclam = strchr (pzargs[i], '!');
+	}
+
       /* Get the system name and file name for this file.  */
       if (zexclam == NULL)
 	{
@@ -599,22 +631,13 @@ main (argc, argv)
 	{
 	  *zexclam = '\0';
 	  zsystem = pzargs[i];
-	  if (*zsystem != '\0')
-	    flocal = FALSE;
-	  else
-	    {
-	      zsystem = zlocalname;
-	      flocal = TRUE;
-	    }
 	  zfile = zexclam + 1;
+	  flocal = FALSE;
 	  zexclam = strrchr (zfile, '!');
 	  if (zexclam == NULL)
 	    zforw = NULL;
 	  else
 	    {
-	      if (flocal)
-		ulog (LOG_FATAL, "!%s: Can't figure out where to get file",
-		      zfile);
 	      *zexclam = '\0';
 	      zforw = zfile;
 	      zfile = zexclam + 1;
@@ -623,8 +646,8 @@ main (argc, argv)
 
       /* Check if the file is already on the execution system.  */
       if (flocal)
-	fonxqt = fxqtlocal;
-      else if (fxqtlocal)
+	fonxqt = fXxqtlocal;
+      else if (fXxqtlocal)
 	fonxqt = FALSE;
       else if (zforward == NULL ? zforw != NULL : zforw == NULL)
 	fonxqt = FALSE;
@@ -632,16 +655,16 @@ main (argc, argv)
 	       && zforw != NULL
 	       && strcmp (zforward, zforw) != 0)
 	fonxqt = FALSE;
-      else if (strcmp (zsystem, sxqtsys.uuconf_zname) == 0)
+      else if (strcmp (zsystem, sXxqtsys.uuconf_zname) == 0)
 	fonxqt = TRUE;
-      else if (sxqtsys.uuconf_pzalias == NULL)
+      else if (sXxqtsys.uuconf_pzalias == NULL)
 	fonxqt = FALSE;
       else
 	{
 	  char **pzal;
 
 	  fonxqt = FALSE;
-	  for (pzal = sxqtsys.uuconf_pzalias; *pzal != NULL; pzal++)
+	  for (pzal = sXxqtsys.uuconf_pzalias; *pzal != NULL; pzal++)
 	    {
 	      if (strcmp (zsystem, *pzal) == 0)
 		{
@@ -653,7 +676,8 @@ main (argc, argv)
 
       /* Turn the file into an absolute path.  */
       if (flocal)
-	zfile = zsysdep_local_file_cwd (zfile, sxqtsys.uuconf_zpubdir);
+	zfile = zsysdep_local_file_cwd (zfile, sXxqtsys.uuconf_zpubdir,
+					(boolean *) NULL);
       else if (fexpand)
 	zfile = zsysdep_add_cwd (zfile);
       if (zfile == NULL)
@@ -665,8 +689,8 @@ main (argc, argv)
 	  if (flocal)
 	    {
 	      if (! fin_directory_list (zfile,
-					sxqtsys.uuconf_pzremote_receive,
-					sxqtsys.uuconf_zpubdir, TRUE,
+					sXxqtsys.uuconf_pzremote_receive,
+					sXxqtsys.uuconf_zpubdir, TRUE,
 					FALSE, (const char *) NULL))
 		ulog (LOG_FATAL, "Not permitted to create %s", zfile);
 	    }
@@ -720,7 +744,7 @@ main (argc, argv)
 	  if (fonxqt)
 	    uxadd_xqt_line ('O', zfile, (const char *) NULL);
 	  else if (flocal)
-	    uxadd_xqt_line ('O', zfile, zxqtloc);
+	    uxadd_xqt_line ('O', zfile, zXxqtloc);
 	  else
 	    uxadd_xqt_line ('O', zfile, zsystem);
 	  pzargs[i] = NULL;
@@ -754,12 +778,12 @@ main (argc, argv)
 	  if (! fsysdep_access (zfile))
 	    uxabort ();
 
-	  zdata = zsysdep_data_file_name (&sxqtsys, zxqtloc, bgrade, FALSE,
+	  zdata = zsysdep_data_file_name (&sXxqtsys, zXxqtloc, bXgrade, FALSE,
 					  abtname, abdname, (char *) NULL);
 	  if (zdata == NULL)
 	    uxabort ();
 
-	  if (fcopy || flink || fxqtlocal)
+	  if (fcopy || flink || fXxqtlocal)
 	    {
 	      boolean fdid;
 
@@ -787,7 +811,7 @@ main (argc, argv)
 		  efile = esysdep_user_fopen (zfile, TRUE, TRUE);
 		  if (! ffileisopen (efile))
 		    uxabort ();
-		  if (! fcopy_open_file (efile, zdata, FALSE, TRUE))
+		  if (! fcopy_open_file (efile, zdata, FALSE, TRUE, TRUE))
 		    uxabort ();
 		  (void) ffileclose (efile);
 		}
@@ -802,8 +826,8 @@ main (argc, argv)
 	      /* Make sure the daemon can access the file.  */
 	      if (! fsysdep_daemon_access (zfile))
 		uxabort ();
-	      if (! fin_directory_list (zfile, sxqtsys.uuconf_pzlocal_send,
-					sxqtsys.uuconf_zpubdir, TRUE,
+	      if (! fin_directory_list (zfile, sXxqtsys.uuconf_pzlocal_send,
+					sXxqtsys.uuconf_zpubdir, TRUE,
 					TRUE, zuser))
 		ulog (LOG_FATAL, "Not permitted to send from %s",
 		      zfile);
@@ -811,7 +835,7 @@ main (argc, argv)
 	      zuse = zfile;
 	    }
 
-	  if (fxqtlocal)
+	  if (fXxqtlocal)
 	    {
 	      if (finput)
 		uxadd_xqt_line ('I', zuse, (char *) NULL);
@@ -834,8 +858,7 @@ main (argc, argv)
 
 		  uxadd_send_file (zuse, abdname,
 				   finputcopied ? "C" : "c",
-				   abtname, zforward, &sxqtsys,
-				   zxqtloc, bgrade);
+				   abtname, zforward);
 		  zbase = zsysdep_base_name (zfile);
 		  if (zbase == NULL)
 		    uxabort ();
@@ -906,7 +929,7 @@ main (argc, argv)
 
 	      /* We must request the file from the remote system to
 		 this one.  */
-	      zdata = zsysdep_data_file_name (&slocalsys, zxqtloc, bgrade,
+	      zdata = zsysdep_data_file_name (&slocalsys, zXxqtloc, bXgrade,
 					      FALSE, abtname, (char *) NULL,
 					      (char *) NULL);
 	      if (zdata == NULL)
@@ -918,6 +941,7 @@ main (argc, argv)
 		 spool directory; normally such requests are rejected.
 		 This privilege is easy to abuse.  */
 	      s.bcmd = 'R';
+	      s.bgrade = bXgrade;
 	      s.pseq = NULL;
 	      s.zfrom = zfile;
 	      s.zto = zbufcpy (abtname);
@@ -930,7 +954,7 @@ main (argc, argv)
 	      s.zcmd = NULL;
 	      s.ipos = 0;
 
-	      zjobid = zsysdep_spool_commands (&sfromsys, bgrade, 1, &s);
+	      zjobid = zsysdep_spool_commands (&sfromsys, bXgrade, 1, &s);
 	      if (zjobid == NULL)
 		uxabort ();
 
@@ -950,7 +974,7 @@ main (argc, argv)
 		  zcall_system = zbufcpy (sfromsys.uuconf_zname);
 		}
 
-	      if (fxqtlocal)
+	      if (fXxqtlocal)
 		{
 		  /* Tell the command execution to wait until the file
 		     has been received, and tell it the real file
@@ -981,8 +1005,8 @@ main (argc, argv)
 		  /* Now we must arrange to forward the file on to the
 		     execution system.  We need to get a name to give
 		     the file on the execution system (abxtname).  */
-		  zdata = zsysdep_data_file_name (&sxqtsys, zxqtloc,
-						  bgrade, TRUE, abxtname,
+		  zdata = zsysdep_data_file_name (&sXxqtsys, zXxqtloc,
+						  bXgrade, TRUE, abxtname,
 						  (char *) NULL,
 						  (char *) NULL);
 		  if (zdata == NULL)
@@ -1004,12 +1028,14 @@ main (argc, argv)
 		  fprintf (e, "U %s %s\n", zsysdep_login_name (),
 			   zlocalname);
 		  fprintf (e, "F %s %s\n", abtname, zbase);
-		  fprintf (e, "C uucp -C -W -d -g %c %s %s!", bgrade,
-			   zbase, sxqtsys.uuconf_zname);
+		  fprintf (e, "C uucp -C -W -d -g %c %s %s!", bXgrade,
+			   zbase, sXxqtsys.uuconf_zname);
 		  if (zforward != NULL)
 		    fprintf (e, "%s!", zforward);
 		  fprintf (e, "%s\n", abxtname);
 
+		  if (! fstdiosync (e, zxqt))
+		    ulog (LOG_FATAL, "fsync failed");
 		  if (fclose (e) != 0)
 		    ulog (LOG_FATAL, "fclose: %s", strerror (errno));
 
@@ -1040,7 +1066,7 @@ main (argc, argv)
       char abdname[CFILE_NAME_LEN];
       FILE *e;
 
-      zdata = zsysdep_data_file_name (&sxqtsys, zxqtloc, bgrade, FALSE,
+      zdata = zsysdep_data_file_name (&sXxqtsys, zXxqtloc, bXgrade, FALSE,
 				      abtname, abdname, (char *) NULL);
       if (zdata == NULL)
 	uxabort ();
@@ -1054,11 +1080,13 @@ main (argc, argv)
 
       uxcopy_stdin (e);
 
+      if (! fstdiosync (e, zdata))
+	ulog (LOG_FATAL, "fsync failed");
       eXclose = NULL;
       if (fclose (e) != 0)
 	ulog (LOG_FATAL, "fclose: %s", strerror (errno));
 
-      if (fxqtlocal)
+      if (fXxqtlocal)
 	uxadd_xqt_line ('I', abtname, (const char *) NULL);
       else
 	{
@@ -1107,17 +1135,19 @@ main (argc, argv)
 	}
     }
 
+  fpoll = FALSE;
+
   /* If we haven't written anything to the execution file yet, and we
      have a standard input file, and we're not forwarding, then every
      other option can be handled in an E command.  */
   if (eXxqt_file == NULL && zinput_from != NULL && zforward == NULL)
     {
       struct scmd s;
-      char aboptions[10];
       char *zoptions;
 
       /* Set up an E command.  */
       s.bcmd = 'E';
+      s.bgrade = bXgrade;
       s.pseq = NULL;
       s.zuser = zuser;
       s.zfrom = zinput_from;
@@ -1147,19 +1177,29 @@ main (argc, argv)
       pasXcmds = (struct scmd *) xrealloc ((pointer) pasXcmds,
 					   cXcmds * sizeof (struct scmd));
       pasXcmds[cXcmds - 1] = s;
+
+      uxadd_name (zinput_from);
+    }
+  else if (*zfullcmd == '\0'
+	   && eXxqt_file == NULL
+	   && zinput_from == NULL
+	   && cXcmds == 0)
+    {
+      /* As a special case, if we are asked to execute an empty
+         command, we create a poll file instead.  */
+      fpoll = TRUE;
     }
   else
     {
       /* Finish up the execute file.  */
-      uxadd_xqt_line ('U', zuser, zxqtloc);
+      uxadd_xqt_line ('U', zuser, zXxqtloc);
       if (zinput_from != NULL)
 	{
 	  uxadd_xqt_line ('F', zinput_to, (char *) NULL);
 	  uxadd_xqt_line ('I', zinput_to, (char *) NULL);
 	  uxadd_send_file (zinput_from, zinput_to,
 			   finputcopied ? "C" : "c",
-			   zinput_temp, zforward, &sxqtsys, zxqtloc,
-			   bgrade);
+			   zinput_temp, zforward);
 	}
       if (fno_ack)
 	uxadd_xqt_line ('N', (const char *) NULL, (const char *) NULL);
@@ -1170,15 +1210,17 @@ main (argc, argv)
       if (fneedshell)
 	uxadd_xqt_line ('e', (const char *) NULL, (const char *) NULL);
       uxadd_xqt_line ('C', zfullcmd, (const char *) NULL);
+      if (! fstdiosync (eXxqt_file, "execution file"))
+	ulog (LOG_FATAL, "fsync failed");
       if (fclose (eXxqt_file) != 0)
 	ulog (LOG_FATAL, "fclose: %s", strerror (errno));
       eXxqt_file = NULL;
 
       /* If the execution is to occur on another system, we must now
 	 arrange to copy the execute file to this system.  */
-      if (! fxqtlocal)
-	uxadd_send_file (abxqt_tname, abxqt_xname, "C", abxqt_tname,
-			 zforward, &sxqtsys, zxqtloc, bgrade);
+      if (! fXxqtlocal)
+	uxadd_send_file (abXxqt_tname, abXxqt_xname, "C", abXxqt_tname,
+			 zforward);
     }
 
   /* If we got a signal, get out before spooling anything.  */
@@ -1186,16 +1228,17 @@ main (argc, argv)
     uxabort ();
 
   /* From here on in, it's too late.  We don't call uxabort.  */
-  if (cXcmds > 0)
+  if (cXcmds > 0 || fpoll)
     {
       char *zjobid;
 
-      if (! sxqtsys.uuconf_fcall_transfer
-	  && ! sxqtsys.uuconf_fcalled_transfer)
+      if (! fpoll
+	  && ! sXxqtsys.uuconf_fcall_transfer
+	  && ! sXxqtsys.uuconf_fcalled_transfer)
 	ulog (LOG_FATAL, "Not permitted to transfer files to or from %s",
-	      sxqtsys.uuconf_zname);
+	      sXxqtsys.uuconf_zname);
 
-      zjobid = zsysdep_spool_commands (&sxqtsys, bgrade, cXcmds, pasXcmds);
+      zjobid = zsysdep_spool_commands (&sXxqtsys, bXgrade, cXcmds, pasXcmds);
       if (zjobid == NULL)
 	{
 	  ulog_close ();
@@ -1215,30 +1258,72 @@ main (argc, argv)
       else
 	{
 	  fcall_any = TRUE;
-	  zcall_system = zbufcpy (sxqtsys.uuconf_zname);
+	  zcall_system = zbufcpy (sXxqtsys.uuconf_zname);
 	}
     }
 
-  /* If all that worked, make a log file entry.  All log file reports
-     up to this point went to stderr.  */
-  ulog_to_file (puuconf, TRUE);
-  ulog_system (sxqtsys.uuconf_zname);
-  ulog_user (zuser);
+  if (! fpoll)
+    {
+      /* If all that worked, make a log file entry.  All log file
+	 reports up to this point went to stderr.  */
+      ulog_to_file (puuconf, TRUE);
+      ulog_system (sXxqtsys.uuconf_zname);
+      ulog_user (zuser);
 
-  ulog (LOG_NORMAL, "Queuing %s", zfullcmd);
+      if (zXnames == NULL)
+	ulog (LOG_NORMAL, "Queuing %s", zfullcmd);
+      else
+	ulog (LOG_NORMAL, "Queuing %s (%s)", zfullcmd, zXnames);
 
-  ulog_close ();
+      ulog_close ();
+    }
 
-  if (! fuucico)
-    fexit = TRUE;
+  if (! fuucico
+      || (zcall_system == NULL && ! fcall_any))
+    {
+      if (! fXxqtlocal || ! fuucico)
+	fexit = TRUE;
+      else
+	{
+	  char *zconfigarg;
+
+	  if (zconfig == NULL)
+	    zconfigarg = NULL;
+	  else
+	    {
+	      zconfigarg = zbufalc (sizeof "-I" + strlen (zconfig));
+	      sprintf (zconfigarg, "-I%s", zconfig);
+	    }
+
+	  fexit = fsysdep_run (FALSE, "uuxqt", zconfigarg,
+			       (const char *) NULL);
+	}
+    }
   else
     {
-      if (zcall_system != NULL)
-	fexit = fsysdep_run ("uucico", "-s", zcall_system);
-      else if (fcall_any)
-	fexit = fsysdep_run ("uucico", "-r1", (const char *) NULL);
+      const char *zcicoarg;
+      char *zconfigarg;
+
+      if (zcall_system == NULL)
+	zcicoarg = "-r1";
       else
-	fexit = TRUE;
+	{
+	  char *z;
+
+	  z = zbufalc (sizeof "-Cs" + strlen (zcall_system));
+	  sprintf (z, "-Cs%s", zcall_system);
+	  zcicoarg = z;
+	}
+
+      if (zconfig == NULL)
+	zconfigarg = NULL;
+      else
+	{
+	  zconfigarg = zbufalc (sizeof "-I" + strlen (zconfig));
+	  sprintf (zconfigarg, "-I%s", zconfig);
+	}
+
+      fexit = fsysdep_run (FALSE, "uucico", zcicoarg, zconfigarg);
     }
 
   usysdep_exit (fexit);
@@ -1249,44 +1334,38 @@ main (argc, argv)
 
 /* Report command usage.  */
 
+static void
+uxhelp ()
+{
+  printf ("Taylor UUCP %s, copyright (C) 1991, 92, 93, 94, 1995 Ian Lance Taylor\n",
+	  VERSION);
+  printf ("Usage: %s [options] [-] command\n", zProgram);
+  printf (" -,-p,--stdin: Read standard input for standard input of command\n");
+  printf (" -c,--nocopy: Do not copy local files to spool directory (default)\n");
+  printf (" -C,--copy: Copy local files to spool directory\n");
+  printf (" -l,--link: link local files to spool directory\n");
+  printf (" -g,--grade grade: Set job grade (must be alphabetic)\n");
+  printf (" -n,--notification=no: Do not report completion status\n");
+  printf (" -z,--notification=error: Report completion status only on error\n");
+  printf (" -r,--nouucico: Do not start uucico daemon\n");
+  printf (" -a,--requestor address: Address to mail status report to\n");
+  printf (" -b,--return-stdin: Return standard input with status report\n");
+  printf (" -s,--status file: Report completion status to file\n");
+  printf (" -j,--jobid: Report job id\n");
+  printf (" -x,--debug debug: Set debugging level\n");
+#if HAVE_TAYLOR_CONFIG
+  printf (" -I,--config file: Set configuration file to use\n");
+#endif /* HAVE_TAYLOR_CONFIG */
+  printf (" -v,--version: Print version and exit\n");
+  printf (" --help: Print help and exit\n");
+}
+
 static void
 uxusage ()
 {
   fprintf (stderr,
-	   "Taylor UUCP version %s, copyright (C) 1991, 1992 Ian Lance Taylor\n",
-	   VERSION);
-  fprintf (stderr,
-	   "Usage: uux [options] [-] command\n");
-  fprintf (stderr,
-	   " -,-p: Read standard input for standard input of command\n");
-  fprintf (stderr,
-	   " -c: Do not copy local files to spool directory (default)\n");
-  fprintf (stderr,
-	   " -C: Copy local files to spool directory\n");
-  fprintf (stderr,
-	   " -l: link local files to spool directory\n");
-  fprintf (stderr,
-	   " -g grade: Set job grade (must be alphabetic)\n");
-  fprintf (stderr,
-	   " -n: Do not report completion status\n");
-  fprintf (stderr,
-	   " -z: Report completion status only on error\n");
-  fprintf (stderr,
-	   " -r: Do not start uucico daemon\n");
-  fprintf (stderr,
-	   " -a address: Address to mail status report to\n");
-  fprintf (stderr,
-	   " -b: Return standard input with status report\n");
-  fprintf (stderr,
-	   " -s file: Report completion status to file\n");
-  fprintf (stderr,
-	   " -j: Report job id\n");
-  fprintf (stderr,
-	   " -x debug: Set debugging level\n");
-#if HAVE_TAYLOR_CONFIG
-  fprintf (stderr,
-	   " -I file: Set configuration file to use\n");
-#endif /* HAVE_TAYLOR_CONFIG */
+	   "Usage: %s [options] [-] command\n", zProgram);
+  fprintf (stderr, "Use %s --help for help\n", zProgram);
   exit (EXIT_FAILURE);
 }
 
@@ -1300,7 +1379,20 @@ uxadd_xqt_line (bchar, z1, z2)
 {
   if (eXxqt_file == NULL)
     {
-      eXxqt_file = esysdep_fopen (zXxqt_name, FALSE, FALSE, TRUE);
+      const char *zxqt_name;
+
+      if (fXxqtlocal)
+	zxqt_name = zsysdep_xqt_file_name ();
+      else
+	zxqt_name = zsysdep_data_file_name (&sXxqtsys, zXxqtloc, bXgrade, TRUE,
+					    abXxqt_tname, (char *) NULL,
+					    abXxqt_xname);
+      if (zxqt_name == NULL)
+	uxabort ();
+
+      uxrecord_file (zxqt_name);
+
+      eXxqt_file = esysdep_fopen (zxqt_name, FALSE, FALSE, TRUE);
       if (eXxqt_file == NULL)
 	uxabort ();
     }
@@ -1316,16 +1408,12 @@ uxadd_xqt_line (bchar, z1, z2)
 /* Add a file to be sent to the execute system.  */
 
 static void
-uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward, qxqtsys, zxqtloc,
-		 bgrade)
+uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward)
      const char *zfrom;
      const char *zto;
      const char *zoptions;
      const char *ztemp;
      const char *zforward;
-     const struct uuconf_system *qxqtsys;
-     const char *zxqtloc;
-     int bgrade;
 {
   struct scmd s;
 
@@ -1345,8 +1433,8 @@ uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward, qxqtsys, zxqtloc,
       if (zbase == NULL)
 	uxabort ();
 
-      zxqt = zsysdep_data_file_name (qxqtsys, zxqtloc, bgrade, TRUE, abtname,
-				     abdname, abxname);
+      zxqt = zsysdep_data_file_name (&sXxqtsys, zXxqtloc, bXgrade, TRUE,
+				     abtname, abdname, abxname);
       if (zxqt == NULL)
 	uxabort ();
       e = esysdep_fopen (zxqt, FALSE, FALSE, TRUE);
@@ -1354,18 +1442,21 @@ uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward, qxqtsys, zxqtloc,
 	uxabort ();
       uxrecord_file (zxqt);
 
-      fprintf (e, "U %s %s\n", zsysdep_login_name (), zxqtloc);
+      fprintf (e, "U %s %s\n", zsysdep_login_name (), zXxqtloc);
       fprintf (e, "F %s %s\n", abdname, zbase);
       fprintf (e, "C uucp -C -W -d -g %c %s %s!%s\n",
-	       bgrade, zbase, zforward, zto);
+	       bXgrade, zbase, zforward, zto);
 
       ubuffree (zbase);
 
+      if (! fstdiosync (e, zxqt))
+	ulog (LOG_FATAL, "fsync failed");
       if (fclose (e) != 0)
 	ulog (LOG_FATAL, "fclose: %s", strerror (errno));
 
       /* Send the execution file.  */
       s.bcmd = 'S';
+      s.bgrade = bXgrade;
       s.pseq = NULL;
       s.zfrom = zbufcpy (abtname);
       s.zto = zbufcpy (abxname);
@@ -1383,12 +1474,15 @@ uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward, qxqtsys, zxqtloc,
 					   cXcmds * sizeof (struct scmd));
       pasXcmds[cXcmds - 1] = s;
 
+      uxadd_name (abtname);
+
       /* Send the data file to abdname where the execution file will
 	 expect it.  */
       zto = abdname;
     }
 
   s.bcmd = 'S';
+  s.bgrade = bXgrade;
   s.pseq = NULL;
   s.zfrom = zbufcpy (zfrom);
   s.zto = zbufcpy (zto);
@@ -1405,6 +1499,8 @@ uxadd_send_file (zfrom, zto, zoptions, ztemp, zforward, qxqtsys, zxqtloc,
   pasXcmds = (struct scmd *) xrealloc ((pointer) pasXcmds,
 				       cXcmds * sizeof (struct scmd));
   pasXcmds[cXcmds - 1] = s;
+
+  uxadd_name (zfrom);
 }
 
 /* Copy stdin to a file.  This is a separate function because it may
@@ -1500,3 +1596,31 @@ uxabort ()
   ulog_close ();
   usysdep_exit (FALSE);
 }
+
+/* Add a name to the list of file names we are going to log.  We log
+   all the file names which will appear in the uucico log file.  This
+   permits people to associate the file send in the uucico log with
+   the uux entry which created the file.  Normally only one file name
+   will appear.  */
+
+static void
+uxadd_name (z)
+     const char *z;
+{
+  if (zXnames == NULL)
+    zXnames = zbufcpy (z);
+  else
+    {
+      size_t cold, cadd;
+      char *znew;
+
+      cold = strlen (zXnames);
+      cadd = strlen (z);
+      znew = zbufalc (cold + 2 + cadd);
+      memcpy (znew, zXnames, cold);
+      znew[cold] = ' ';
+      memcpy (znew + cold + 1, z, cadd + 1);
+      ubuffree (zXnames);
+      zXnames = znew;
+    }
+}
